@@ -28,6 +28,10 @@
 #' @param clusteralg Character string: clustering algorithm for eigenvector matrix (GMM or km)
 #' @param FASP Logical flag: whether to use Fast Approximate Spectral Clustering (for v. high sample numbers)
 #' @param FASPk Numerical value: the number of centroids to compute when doing FASP
+#' @param krangemax Numerical value: the maximum K value to iterate towards when running a range of K
+#' @param runrange Logical flag: whether to run a range of K or not (default=FALSE), puts Kth results into Kth element of list
+#' @param diffusion_iters Numerical value: number of diffusion iterations for the graph (default=5)
+#' @param KNNs_p Numerical value: number of KNNs when making KNN graph (default=10)
 #'
 #' @return A list, containing: 
 #' 1) cluster assignments, in the same order as input data columns 
@@ -45,7 +49,8 @@ Spectrum <- function(data,method=1,silent=FALSE,showres=TRUE,diffusion=TRUE,
                      showpca=FALSE,showheatmap=FALSE,showdimred=FALSE,
                      visualisation=c('umap','tsne'),frac=2,thresh=7,
                      fontsize=18,dotsize=3,tunekernel=TRUE,clusteralg='GMM',
-                     FASP=FALSE,FASPk=NULL,fixk=NULL){
+                     FASP=FALSE,FASPk=NULL,fixk=NULL,krangemax=10,
+                     runrange=FALSE,diffusion_iters=4,KNNs_p=10){
   
   ###
   kerneltype <- match.arg(kerneltype)
@@ -63,6 +68,12 @@ Spectrum <- function(data,method=1,silent=FALSE,showres=TRUE,diffusion=TRUE,
   if (is.null(FASPk) == TRUE & FASP == TRUE){
     stop("Error: FASP method requires a number of centroids to compute")
   }
+  if (runrange == TRUE & method == 3){
+    stop("Error: cannot run a range of K whilst method=3")
+  }
+  if (is.null(fixk) == TRUE & method == 3){
+    stop("Error: need to set the value of K using the fixk parameter for method 3")
+  }
   
   ### initial messages
   if (silent == FALSE){
@@ -130,7 +141,7 @@ Spectrum <- function(data,method=1,silent=FALSE,showres=TRUE,diffusion=TRUE,
   if (diffusion == TRUE){
     ## get KNN graph
     for (col in seq(1,ncol(A))){
-      KNNs <- head(rev(sort(A[,col])),(10+1)) # find the KNNs (10 default)
+      KNNs <- head(rev(sort(A[,col])),(KNNs_p+1)) # find the KNNs (10 default)
       tokeep <- names(KNNs)
       A[!(names(A[,col])%in%tokeep),col] <- 0
     }
@@ -146,7 +157,7 @@ Spectrum <- function(data,method=1,silent=FALSE,showres=TRUE,diffusion=TRUE,
     im <- matrix(ncol=ncol(A),nrow=ncol(A))
     im[is.na(im)] <- 0
     diag(im) <- 1
-    for (t in seq(2,5)){ # (5)
+    for (t in seq(1,diffusion_iters)){ # diffusion_iterations (4 default)
       Qt <- A%*%Qt%*%t(A)+im
     }
     A2 <- t(Qt)
@@ -214,89 +225,152 @@ Spectrum <- function(data,method=1,silent=FALSE,showres=TRUE,diffusion=TRUE,
     optk <- fixk
   } 
   
-  ### select optimal eigenvectors
-  xi <- decomp$vectors[,1:optk]
-  # normalise rows
-  yi <- xi/sqrt(rowSums(xi^2))
-  # replace NA values (row = 0) with zeros
-  yi[which(!is.finite(yi))] <- 0
-  
-  if (clusteralg == 'GMM'){
-    ### GMM
-    if (silent == FALSE){
-      message('doing GMM clustering...')
+  ## cluster either a range of K or just optimal K
+  results <- list()
+  if (runrange){
+    for (tk in seq(2,krangemax)){
+      # get range of k results
+      ### select optimal eigenvectors
+      xi <- decomp$vectors[,1:tk]
+      # normalise rows
+      yi <- xi/sqrt(rowSums(xi^2))
+      # replace NA values (row = 0) with zeros
+      yi[which(!is.finite(yi))] <- 0
+      
+      if (clusteralg == 'GMM'){
+        ### GMM
+        #if (silent == FALSE){
+        #  message('doing GMM clustering...')
+        #}
+        gmm <- ClusterR::GMM(yi, tk, verbose = F, seed_mode = "random_spread") # use random spread          
+        pr <- ClusterR::predict_GMM(yi, gmm$centroids, gmm$covariance_matrices, gmm$weights)
+        names(pr)[3] <- 'cluster'
+        if (0 %in% pr$cluster){
+          pr$cluster <- pr$cluster+1
+        }
+        if (silent == FALSE){
+          message('clustered.')
+        }
+      }else if (clusteralg == 'km'){
+        ### k means
+        #if (silent == FALSE){
+        #  message('doing k means clustering...')
+        #}
+        pr <- kmeans(yi, tk)
+        if (silent == FALSE){
+          message('clustered.')
+        }
+      }
+      #
+      if (showres == TRUE){ # for any number of data sources
+        if (showheatmap == TRUE){
+          displayClusters(A2,group=pr$cluster,fsize=1)
+        }
+      }
+      #
+      if (method != 3){
+        if (FASP){
+          ### if running FASP assign original samples to centroid clusters
+          casn <- cas
+          casn <- casn[seq_along(casn)] <- pr$cluster[as.numeric(casn[seq_along(casn)])]
+          names(casn) <- names(cas) 
+          ### return results
+          results[[tk]] <- list('allsample_assignments'=casn,'centroid_assignments'=pr$cluster,
+                          'eigenvector_analysis'=d,'K'=tk,'similarity_matrix'=A2,
+                          'eigendecomposition'=decomp)
+        }else{
+          ### return results
+          results[[tk]] <- list('assignments'=pr$cluster,'eigenvector_analysis'=d,
+                          'K'=tk,'similarity_matrix'=A2,'eigendecomposition'=decomp)
+        }
+      }
     }
-    gmm <- ClusterR::GMM(yi, optk, verbose = F, seed_mode = "random_spread") # use random spread          
-    pr <- ClusterR::predict_GMM(yi, gmm$centroids, gmm$covariance_matrices, gmm$weights)
-    names(pr)[3] <- 'cluster'
-    if (0 %in% pr$cluster){
-      pr$cluster <- pr$cluster+1
+  }else{
+    # just optimal k/ forced k
+    ### select optimal eigenvectors
+    xi <- decomp$vectors[,1:optk]
+    # normalise rows
+    yi <- xi/sqrt(rowSums(xi^2))
+    # replace NA values (row = 0) with zeros
+    yi[which(!is.finite(yi))] <- 0
+    
+    if (clusteralg == 'GMM'){
+      ### GMM
+      if (silent == FALSE){
+        message('doing GMM clustering...')
+      }
+      gmm <- ClusterR::GMM(yi, optk, verbose = F, seed_mode = "random_spread") # use random spread          
+      pr <- ClusterR::predict_GMM(yi, gmm$centroids, gmm$covariance_matrices, gmm$weights)
+      names(pr)[3] <- 'cluster'
+      if (0 %in% pr$cluster){
+        pr$cluster <- pr$cluster+1
+      }
+      if (silent == FALSE){
+        message('done.')
+      }
+    }else if (clusteralg == 'km'){
+      ### k means
+      if (silent == FALSE){
+        message('doing k means clustering...')
+      }
+      pr <- kmeans(yi, optk)
+      if (silent == FALSE){
+        message('done.')
+      }
     }
-    if (silent == FALSE){
-      message('done.')
+    
+    ### display clusters using heatmap and tsne
+    if (showres == TRUE){ # for any number of data sources
+      if (showheatmap == TRUE){
+        displayClusters(A2,group=pr$cluster,fsize=1)
+      }
+      if (showdimred == TRUE && visualisation == 'umap'){ # method 1
+        message('running UMAP on similarity...')
+        umap(A2,labels=as.factor(pr$cluster),axistextsize=fontsize,legendtextsize=fontsize,dotsize=dotsize)
+        message('done.')
+      }
+      if (showdimred == TRUE && visualisation == 'tsne'){ # method 2
+        message('running t-SNE on similarity...')
+        tsne(A2,labels=as.factor(pr$cluster),axistextsize=fontsize,legendtextsize=fontsize,dotsize=dotsize)
+        message('done.')
+      }
     }
-  }else if (clusteralg == 'km'){
-    ### k means
-    if (silent == FALSE){
-      message('doing k means clustering...')
+    if (length(datalist) == 1 && showres == TRUE){ # for one data source only
+      if (showpca == TRUE){
+        pca(datalist[[1]],labels=as.factor(pr$cluster),axistextsize=fontsize,legendtextsize=fontsize,dotsize=dotsize)
+      }
     }
-    pr <- kmeans(yi, optk)
-    if (silent == FALSE){
-      message('done.')
-    }
-  }
-  
-  ### display clusters using heatmap and tsne
-  if (showres == TRUE){ # for any number of data sources
-    if (showheatmap == TRUE){
-      displayClusters(A2,group=pr$cluster,fsize=1)
-    }
-    if (showdimred == TRUE && visualisation == 'umap'){ # method 1
-      message('running UMAP on similarity...')
-      umap(A2,labels=as.factor(pr$cluster),axistextsize=fontsize,legendtextsize=fontsize,dotsize=dotsize)
-      message('done.')
-    }
-    if (showdimred == TRUE && visualisation == 'tsne'){ # method 2
-      message('running t-SNE on similarity...')
-      tsne(A2,labels=as.factor(pr$cluster),axistextsize=fontsize,legendtextsize=fontsize,dotsize=dotsize)
-      message('done.')
-    }
-  }
-  if (length(datalist) == 1 && showres == TRUE){ # for one data source only
-    if (showpca == TRUE){
-      pca(datalist[[1]],labels=as.factor(pr$cluster),axistextsize=fontsize,legendtextsize=fontsize,dotsize=dotsize)
-    }
-  }
-  
-  if (method != 3){
-    if (FASP){
-      ### if running FASP assign original samples to centroid clusters
-      casn <- cas
-      casn <- casn[seq_along(casn)] <- pr$cluster[as.numeric(casn[seq_along(casn)])]
-      names(casn) <- names(cas) 
-      ### return results
-      results <- list('allsample_assignments'=casn,'centroid_assignments'=pr$cluster,
-                      'eigenvector_analysis'=d,'K'=optk,'similarity_matrix'=A2,
-                      'eigendecomposition'=decomp)
-    }else{
-      ### return results
-      results <- list('assignments'=pr$cluster,'eigenvector_analysis'=d,
-                      'K'=optk,'similarity_matrix'=A2,'eigendecomposition'=decomp)
-    }
-  }else if (method == 3){
-    if (FASP){
-      ### if running FASP assign original samples to centroid clusters
-      casn <- cas
-      casn <- casn[seq_along(casn)] <- pr$cluster[as.numeric(casn[seq_along(casn)])]
-      names(casn) <- names(cas) 
-      ### return results
-      results <- list('allsample_assignments'=casn,'centroid_assignments'=pr$cluster,
-                      'K'=optk,'similarity_matrix'=A2,
-                      'eigendecomposition'=decomp)
-    }else{
-      ### return results
-      results <- list('assignments'=pr$cluster,
-                      'K'=optk,'similarity_matrix'=A2,'eigendecomposition'=decomp)
+    
+    if (method != 3){
+      if (FASP){
+        ### if running FASP assign original samples to centroid clusters
+        casn <- cas
+        casn <- casn[seq_along(casn)] <- pr$cluster[as.numeric(casn[seq_along(casn)])]
+        names(casn) <- names(cas) 
+        ### return results
+        results <- list('allsample_assignments'=casn,'centroid_assignments'=pr$cluster,
+                        'eigenvector_analysis'=d,'K'=optk,'similarity_matrix'=A2,
+                        'eigendecomposition'=decomp)
+      }else{
+        ### return results
+        results <- list('assignments'=pr$cluster,'eigenvector_analysis'=d,
+                        'K'=optk,'similarity_matrix'=A2,'eigendecomposition'=decomp)
+      }
+    }else if (method == 3){
+      if (FASP){
+        ### if running FASP assign original samples to centroid clusters
+        casn <- cas
+        casn <- casn[seq_along(casn)] <- pr$cluster[as.numeric(casn[seq_along(casn)])]
+        names(casn) <- names(cas) 
+        ### return results
+        results <- list('allsample_assignments'=casn,'centroid_assignments'=pr$cluster,
+                        'K'=optk,'similarity_matrix'=A2,
+                        'eigendecomposition'=decomp)
+      }else{
+        ### return results
+        results <- list('assignments'=pr$cluster,
+                        'K'=optk,'similarity_matrix'=A2,'eigendecomposition'=decomp)
+      }
     }
   }
   
@@ -306,5 +380,4 @@ Spectrum <- function(data,method=1,silent=FALSE,showres=TRUE,diffusion=TRUE,
   
   return(results)
 }
-  
 
