@@ -18,23 +18,41 @@ test2 <- Spectrum(RNAseq,showdimred=TRUE,fontsize=8,dotsize=2)
 
 ## ----fig.width=4.5,fig.height=3------------------------------------------
 library(Spectrum)
-test3 <- Spectrum(brain,showdimred=TRUE,fontsize=8,dotsize=2)
-
-## ----fig.width=4.5,fig.height=3------------------------------------------
-library(Spectrum)
-test4 <- Spectrum(circles,showpca=TRUE,method=2,fontsize=8,dotsize=2)
-
-## ----fig.width=4.5,fig.height=3------------------------------------------
-library(Spectrum)
-test5 <- Spectrum(spirals,showpca=TRUE,method=2,fontsize=8,dotsize=2)
-
-## ----fig.width=4.5,fig.height=3------------------------------------------
-library(Spectrum)
-test6 <- Spectrum(blobs,FASP=TRUE,FASPk=300,fontsize=8,dotsize=2)
+RNAseq <- brain[[1]]
+test3 <- Spectrum(RNAseq,showres=FALSE,runrange=TRUE,krangemax=10)
 
 ## ------------------------------------------------------------------------
-names(test6)
+head(test3[[2]]$assignments)
+
+## ----fig.width=4.5,fig.height=3------------------------------------------
+library(Spectrum)
+test4 <- Spectrum(brain,showdimred=TRUE,fontsize=8,dotsize=2)
+
+## ----fig.width=4.5,fig.height=3------------------------------------------
+library(Spectrum)
+test5 <- Spectrum(circles,showpca=TRUE,method=2,fontsize=8,dotsize=2)
+
+## ----fig.width=4.5,fig.height=3------------------------------------------
+library(Spectrum)
+test6 <- Spectrum(spirals,showpca=TRUE,method=2,fontsize=8,dotsize=2)
+
+## ----fig.width=4.5,fig.height=3------------------------------------------
+library(Spectrum)
+test7 <- Spectrum(blobs,FASP=TRUE,FASPk=300,fontsize=8,dotsize=2)
 
 ## ------------------------------------------------------------------------
-head(test6[[1]])
+names(test7)
+
+## ------------------------------------------------------------------------
+head(test7[[1]])
+
+## ------------------------------------------------------------------------
+## 1. run my clustering algorithm yielding assignments, e.g. 1,2,2,1,2,2,1,2,1,2
+# algorithm function, e.g. Spectrum
+## 2. reorder data according to any clustering algorithm
+ind <- sort(as.vector(test2$assignments),index.return=TRUE) ## get the indices required for sorting
+datax <- RNAseq[,ind$ix] ## order the original data according to the clustering assignments
+#annonx <- meta[ind$ix,] ## order the meta data for the heatmap function
+#annonx$cluster <- ind$x ## add the cluster to the meta data
+## 3. do heatmap your heatmap
 
